import math
import bpy
import bpy.types
import logging
import bmesh


# Thanks
# https://github.com/nutti/Magic-UV/blob/master/src/magic_uv/common.py#L1243
def get_uv_editable_objects(context):
    if compat.check_version(2, 80, 0) < 0:
        objs = []
    else:
        objs = [
            o for o in bpy.data.objects
            if compat.get_object_select(o) and o.type == 'MESH'
        ]

    ob = context.active_object
    if ob is not None:
        objs.append(ob)

    objs = list(set(objs))
    return objs


class AlignIsland(bpy.types.Operator):
    """
    Utility to align a UV island to grid direction based on the selected edge.
    """

    bl_idname = "uv.solstice_align_island"
    bl_label = "Align Island"
    bl_options = {"REGISTER", "UNDO"}

    def execute(self, context):
        self.report({"INFO"}, "Aligning UV island")

        objs = get_uv_editable_objects(context)
        for obj in objs:

            bm = bmesh.from_edit_mesh(obj.data)
            if common.check_version(2, 73, 0) >= 0:
                bm.faces.ensure_lookup_table()
            uv_layer = bm.loops.layers.uv.verify()

            # Get the selected UV edge
            selected_edges = []
            for edge, selected in uv_map.edge_selection.items():
                if selected.value:
                    selected_edges.append(edge)

            if len(selected_edges) != 1:
                self.report(
                    {"ERROR"},
                    "Please select a single edge. Got: {} edge(s).".format(
                        len(selected_edges)))
                return {"CANCELLED"}

            # Get the UV island that the edge belongs to
            selected_edge = selected_edges[0]
            self.report({"INFO"}, f"Selected edge: {selected_edge.index}")

            selected_island = None
            for island in uv_map.data.islands:
                if selected_edge in island.edges:
                    selected_island = island
                    break

            if selected_island is None:
                self.report({"ERROR"}, "Could not find UV island")
                return {"CANCELLED"}

            # Get the UV coordinates of the selected edge
            selected_edge_uv = []
            for vert in selected_edge.vertices:
                selected_edge_uv.append(selected_island.uv[vert].uv)

            # Check the direction of the edge in UV space as degrees
            edge_direction = (selected_edge_uv[1] -
                              selected_edge_uv[0]).to_track_quat(
                                  "X", "Y").to_euler().z

            # Choose the closest grid direction
            if edge_direction < 45 or edge_direction > 315:
                grid_direction = 0
            elif edge_direction < 135:
                grid_direction = 90
            elif edge_direction < 225:
                grid_direction = 180
            else:
                grid_direction = 270

            # Calculate the rotation needed to align the island to the grid direction
            rotation = grid_direction - edge_direction

            self.report({"INFO"},
                        f"Edge direction: {math.degrees(edge_direction)}")
            self.report({"INFO"}, f"Grid direction: {grid_direction}")

            # Select the island
            bpy.ops.uv.select_linked()

            bpy.ops.uv.loop_rotate(radians=math.radians(rotation))

        return {"FINISHED"}
